@@ -34,6 +34,7 @@ editMap <- function(x, ...) {
 #' @param crs see \code{\link[sf]{st_crs}}.
 #' @param title \code{string} to customize the title of the UI window.  The default
 #'          is "Edit Map".
+#' @param editor \code{character} either "leaflet.extras" or "leafpm"
 #'
 #' @details
 #'   When setting \code{viewer = browserViewer(browser = getOption("browser"))} and
@@ -51,6 +52,7 @@ editMap.leaflet <- function(
   ns = "mapedit-edit", record = FALSE, viewer = shiny::paneViewer(),
   crs = 4326,
   title = "Edit Map",
+  editor = c("leaflet.extras", "leafpm"),
   ...
 ) {
   stopifnot(!is.null(x), inherits(x, "leaflet"))
@@ -72,7 +74,7 @@ editMap.leaflet <- function(
       right = miniUI::miniTitleBarButton("done", "Done", primary = TRUE)
     ),
     tags$script(HTML(
-"
+      "
 // close browser window on session end
 $(document).on('shiny:disconnected', function() {
   // check to make sure that button was pressed
@@ -96,7 +98,8 @@ $(document).on('shiny:disconnected', function() {
       targetLayerId = targetLayerId,
       sf = sf,
       record = record,
-      crs = crs
+      crs = crs,
+      editor = editor
     )
 
     observe({crud()})
@@ -136,6 +139,7 @@ editMap.mapview <- function(
   ns = "mapedit-edit", record = FALSE, viewer = shiny::paneViewer(),
   crs = 4326,
   title = "Edit Map",
+  editor = c("leaflet.extras", "leafpm"),
   ...
 ) {
   stopifnot(!is.null(x), inherits(x, "mapview"), inherits(x@map, "leaflet"))
@@ -143,13 +147,14 @@ editMap.mapview <- function(
   editMap.leaflet(
     x@map, targetLayerId = targetLayerId, sf = sf,
     ns = ns, viewer = viewer, record = TRUE, crs = crs,
-    title = title
+    title = title,
+    editor = editor
   )
 }
 
 #' @name editMap
 #' @export
-editMap.NULL = function(x, ...) {
+editMap.NULL = function(x, editor = c("leaflet.extras", "leafpm"), ...) {
   m = mapview::mapview()@map
   m = leaflet::fitBounds(
     m,
@@ -158,7 +163,7 @@ editMap.NULL = function(x, ...) {
     lng2 = 180, #as.numeric(sf::st_bbox(x)[3]),
     lat2 = 90 #as.numeric(sf::st_bbox(x)[4])
   )
-  ed = editMap(m, record=TRUE)
+  ed = editMap(m, record = TRUE, editor = editor)
   ed_record <- ed$finished
   attr(ed_record, "recorder") <- attr(ed, "recorder", exact = TRUE)
   ed_record
@@ -196,6 +201,7 @@ editFeatures = function(x, ...) {
 #' @param crs see \code{\link[sf]{st_crs}}.
 #' @param title \code{string} to customize the title of the UI window.  The default
 #'          is "Edit Map".
+#' @param editor \code{character} either "leaflet.extras" or "leafpm"
 #'
 #' @details
 #'   When setting \code{viewer = browserViewer(browser = getOption("browser"))} and
@@ -217,9 +223,19 @@ editFeatures.sf = function(
   crs = 4326,
   label = NULL,
   title = "Edit Map",
+  editor = c("leaflet.extras", "leafpm"),
   ...
 ) {
 
+  # store original projection of edited object ----
+  orig_proj <- sf::st_crs(x)
+  if (is.na(orig_proj)) {
+    stop("The CRS of the input object is not set. Aborting. `mapedit` does not currently
+         allow editing objects with arbitrary coordinates system. Please set the
+         CRS of the input using `sf::st_set_crs()` (for `sf` objects) or `proj4string()
+         for `sp` objects", call. = FALSE)
+  }
+
   x$edit_id = as.character(1:nrow(x))
 
   if (is.null(map)) {
@@ -252,10 +268,27 @@ editFeatures.sf = function(
     )
   }
 
+  # currently we don't have a way to set custom options for leaflet.pm
+  # and we will want to customize allowSelfIntersection depending on feature types
+  if(inherits(map, "mapview")) map = map@map
+  if(editor[1] == "leafpm") {
+    # now let's see if any of the features are polygons
+    if(any(sf::st_dimension(x) == 2)) {
+      map = leafpm::addPmToolbar(
+        map,
+        targetGroup = "toedit",
+        toolbarOptions = leafpm::pmToolbarOptions(drawCircle = FALSE),
+        drawOptions = leafpm::pmDrawOptions(allowSelfIntersection = FALSE),
+        editOptions = leafpm::pmEditOptions(allowSelfIntersection = FALSE),
+        cutOptions = leafpm::pmCutOptions(allowSelfIntersection = FALSE)
+      )
+    }
+  }
+
   crud = editMap(
     map, targetLayerId = "toedit",
     viewer = viewer, record = record,
-    crs = crs, title = title, ...
+    crs = crs, title = title, editor = editor, ...
   )
 
   merged <- Reduce(
@@ -293,6 +326,16 @@ editFeatures.sf = function(
 
   merged <- dplyr::select_(merged, "-edit_id")
 
+  # re-transform to original projection if needed ----
+  if (sf::st_crs(merged) != orig_proj) {
+    merged <- sf::st_transform(merged, orig_proj)
+  }
+
+  # warn if anything is not valid
+  if(!all(sf::st_is_valid(merged))) {
+    warning("returned features do not appear valid; please inspect closely", call. = FALSE)
+  }
+
   # return merged features
   if(record==TRUE) {
     attr(merged, "recorder") <- attr(crud, "recorder", exact=TRUE)
