@@ -98,6 +98,7 @@ editModUI <- function(id, ...) {
 #'          \code{GeoJSON}.
 #' @param record \code{logical} to record all edits for future playback.
 #' @param crs see \code{\link[sf]{st_crs}}.
+#' @param editor \code{character} either "leaflet.extras" or "leafpm"
 #'
 #' @return server function for Shiny module
 #' @import shiny
@@ -108,29 +109,40 @@ editMod <- function(
   targetLayerId = NULL,
   sf = TRUE,
   record = FALSE,
-  crs = 4326
+  crs = 4326,
+  editor = c("leaflet.extras", "leafpm")
 ) {
   # check to see if addDrawToolbar has been already added to the map
   if(is.null(
     Find(
       function(cl) {
-        cl$method == "addDrawToolbar"
+        cl$method == "addDrawToolbar" || cl$method == "addPmToolbar"
       },
       leafmap$x$calls
     )
   )) {
-    # add draw toolbar if not found
-    leafmap <- leaflet.extras::addDrawToolbar(
-      leafmap,
-      targetGroup = targetLayerId,
-      polylineOptions = leaflet.extras::drawPolylineOptions(repeatMode = TRUE),
-      polygonOptions = leaflet.extras::drawPolygonOptions(repeatMode = TRUE),
-      circleOptions = FALSE,
-      rectangleOptions = leaflet.extras::drawRectangleOptions(repeatMode = TRUE),
-      markerOptions = leaflet.extras::drawMarkerOptions(repeatMode = TRUE),
-      circleMarkerOptions = leaflet.extras::drawCircleMarkerOptions(repeatMode = TRUE),
-      editOptions = leaflet.extras::editToolbarOptions()
-    )
+    if(editor[1] == "leaflet.extras") {
+      # add draw toolbar if not found
+      leafmap <- leaflet.extras::addDrawToolbar(
+        leafmap,
+        targetGroup = targetLayerId,
+        polylineOptions = leaflet.extras::drawPolylineOptions(repeatMode = TRUE),
+        polygonOptions = leaflet.extras::drawPolygonOptions(repeatMode = TRUE),
+        circleOptions = FALSE,
+        rectangleOptions = leaflet.extras::drawRectangleOptions(repeatMode = TRUE),
+        markerOptions = leaflet.extras::drawMarkerOptions(repeatMode = TRUE),
+        circleMarkerOptions = leaflet.extras::drawCircleMarkerOptions(repeatMode = TRUE),
+        editOptions = leaflet.extras::editToolbarOptions()
+      )
+    }
+
+    if(editor[1] == "leafpm") {
+      leafmap <- leafpm::addPmToolbar(
+        leafmap,
+        targetGroup = targetLayerId,
+        toolbarOptions = leafpm::pmToolbarOptions(drawCircle = FALSE)
+      )
+    }
   }
 
   output$map <- leaflet::renderLeaflet({leafmap})
@@ -174,9 +186,20 @@ editMod <- function(
 
   shiny::observeEvent(input[[EVT_DELETE]], {
     deleted <- input[[EVT_DELETE]]
+
     # find the deleted features and update finished
     # start by getting the leaflet ids to do the match
     ids <- unlist(lapply(featurelist$finished, function(x){x$properties$`_leaflet_id`}))
+
+    # leaflet.pm returns only a single feature while leaflet.extras returns feature collection
+    # convert leaflet.pm so logic will be the same
+    if(editor == "leafpm") {
+      deleted <- list(
+        type = "FeatureCollection",
+        features = list(deleted)
+      )
+    }
+
     # now modify finished to match edited
     lapply(deleted$features, function(x) {
       loc <- match(x$properties$`_leaflet_id`, ids)
